package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
	"regexp"
)

type OrgUserFilter struct {
	Organization *OrganizationID
	Search       *string
}

func (f OrgUserFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"deleted": nil}}})

	if f.Organization != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"organizationID": *f.Organization}}})
	}

	if f.Search != nil {
		ss := ".*" + regexp.QuoteMeta(*f.Search) + ".*"
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{
			"$or": bson.A{
				bson.M{"username": bson.M{"$regex": ss, "$options": "i"}},
				bson.M{"email": bson.M{"$regex": ss, "$options": "i"}},
				bson.M{"firstName": bson.M{"$regex": ss, "$options": "i"}},
				bson.M{"lastName": bson.M{"$regex": ss, "$options": "i"}},
			},
		}}})
	}

	return pipeline
}

func (f OrgUserFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "creationTime", ct.SortDESC, "_id", ct.SortDESC
}
